/* 
 * Copyright (C) 2000 the xine project
 * 
 * This file is part of xine, a unix video player.
 * 
 * xine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * xine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: gui_slider.c,v 1.4 2000/11/14 03:18:58 guenter Exp $
 *
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include "gui_widget.h"
#include "gui_slider.h"
#include "gui_widget_types.h"
#include "gui_main.h"

extern Display *gDisplay;

void paint_slider (widget_t *b,  Window win, GC gc) {
  float tmp;
	
  slider_private_data_t *private_data = (slider_private_data_t *) b->private_data;
  gui_image_t *bg = (gui_image_t *) private_data->bg;
  gui_image_t *paddle = (gui_image_t *) private_data->paddle;

  XLockDisplay (gDisplay);
  
  if (b->widget_type == WIDGET_TYPE_SLIDER) {

    if (private_data->bArmed) {
       if (private_data->bClicked) {
	 XCopyArea (gDisplay, bg->image, win, gc, 0, 0,
		    bg->width, bg->height, b->x, b->y);
		    
       } else {
	 XCopyArea (gDisplay, bg->image, win, gc, 0, 0,
		    bg->width, bg->height, b->x, b->y);
       }
    } else {
	 XCopyArea (gDisplay, bg->image, win, gc, 0, 0,
		    bg->width, bg->height, b->x, b->y);
    }
    XSetForeground(gDisplay, gc, gui_color.white.pixel);
    
    tmp = bg->width - paddle->width;
    tmp = tmp / private_data->max;
    tmp = tmp * private_data->pos;
    XCopyArea (gDisplay, paddle->image, win, gc, 0, 0,
		    paddle->width, paddle->height, 
		    b->x+(int)tmp, 
		    b->y);

    XFlush (gDisplay);

  } else
    fprintf (stderr, "paint slider on something (%d) that is not a slider\n",
	     b->widget_type);

  XUnlockDisplay (gDisplay);

}

int notify_click_slider (widget_t *b,int bUp) {
  slider_private_data_t *private_data = (slider_private_data_t *) b->private_data;
  
  if (b->widget_type == WIDGET_TYPE_SLIDER) {
    private_data->bClicked = !bUp;
  } else
    fprintf (stderr, "notify click slider on something (%d) that is not a slider\n",
	     b->widget_type);
  return 1;
}

int notify_focus_slider (widget_t *b,int bEntered) {
  slider_private_data_t *private_data = (slider_private_data_t *) b->private_data;
  
  if (b->widget_type == WIDGET_TYPE_SLIDER) {

    private_data->bArmed = bEntered;
    
  } else
    fprintf (stderr, "notify slider button on something (%d) that is not a slider\n",
	     b->widget_type);
  return 1;
}

widget_t *create_slider (int x, int y, int w, int h, void* f, int max, int step, const char bg[], const char paddle[]) {
  widget_t              *mywidget;
  slider_private_data_t *private_data;

  mywidget = (widget_t *) malloc (sizeof(widget_t));

  mywidget->x            = x;
  mywidget->y            = y;
  mywidget->width        = w;
  mywidget->height       = h;
  mywidget->widget_type  = WIDGET_TYPE_SLIDER;
  mywidget->paint        = paint_slider;
  mywidget->notify_click = notify_click_slider;
  mywidget->notify_focus = notify_focus_slider;

  private_data = (slider_private_data_t *) malloc (sizeof (slider_private_data_t));

  private_data->bClicked = 0;
  private_data->bArmed   = 0;
  //private_data->min      = min;  // needed ??? see also slider_set_min(...); 
  private_data->max      = max;
  private_data->pos      = 0;      // ... = min;
  private_data->step     = step;
  
  private_data->paddle   = gui_load_image(gui_get_skindir(paddle));
  private_data->bg       = gui_load_image(gui_get_skindir(bg));

  mywidget->private_data = private_data;

  return mywidget;
}

void slider_set_pos(widget_t *b, int pos)
{
  slider_private_data_t *private_data = (slider_private_data_t *) b->private_data;

  if (b->widget_type == WIDGET_TYPE_SLIDER) {
    if (pos >= 0 && pos <= private_data->max) //if (pos >= private_data->min && ....)
  	private_data->pos = pos;
    else
	private_data->pos = 0; //private_data->min; ???
  } else
    fprintf (stderr, "slider_set_pos on something (%d) that is not a slider\n",
		             b->widget_type);
  
}

int slider_get_pos(widget_t *b)
{
  slider_private_data_t *private_data = (slider_private_data_t *) b->private_data;

  if (b->widget_type == WIDGET_TYPE_SLIDER) {
    return private_data->pos;
  } else
    fprintf (stderr, "slider_get_pos on something (%d) that is not a slider\n", b->widget_type);
  return -1;
}

void slider_make_step(widget_t *b)
{
  slider_private_data_t *private_data = (slider_private_data_t *) b->private_data;

  if (b->widget_type == WIDGET_TYPE_SLIDER) {
    slider_set_pos(b, slider_get_pos(b) + private_data->step);
  } else
    fprintf (stderr, "slider_make_step on something (%d) that is not a slider\n", b->widget_type);
}


void slider_set_min(widget_t *b)
{
  slider_private_data_t *private_data = (slider_private_data_t *) b->private_data;

  if (b->widget_type == WIDGET_TYPE_SLIDER) {
    private_data->pos = 0;  //private_data->pos = min; ???
  } else
    fprintf(stderr, "slider_set_min on something (%d) that is not a slider\n", b->widget_type);
}
