/* 
 * Copyright (C) 2000 the xine project
 * 
 * This file is part of xine, a unix video player.
 * 
 * xine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * xine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: input_dvd.c,v 1.6 2000/11/15 18:05:48 guenter Exp $
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <dlfcn.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>

#include "input_plugin.h"
#include "dvd_udf.h"

#define DVD     "/dev/dvd"

/*
 * global Variables:
 */

static int                file_fd;
static int                file_size, file_size_left;
static int                file_lbstart, file_lbcur;

/*
 * udf dir function 
 */

#define MAX_DIR_ENTRIES 250

static char             **flist;
static int                nFiles;



/*
 * try to open dvd and prepare to read >filename<
 *
 * returns lbnum on success, 0 otherwise
 */

static int openDVDFile (char *filename, uint32_t *size) {

  char               str[256];
  int                lbnum;

  printf ("input_dvd : openDVDFile >%s<\n",filename);

  if ((file_fd=UDFOpenDisc(DVD)) >= 0) {

    sprintf (str, "/VIDEO_TS/%s", filename);

    printf ("UDFFindFile %s\n",str);

    if (!(lbnum=UDFOpenFile(str, size))) {
      printf ("input_dvd: cannot open file >%s<\n", filename);
      return 0;
    }
  } else {
    printf ("input_dvd: cannot open dvd drive >%s<\n", DVD);
    return 0;
  }

  return lbnum;
}


static void input_plugin_init (void) {
  int i;
  
  /*
   * allocate space for directory listing
   */

  flist = (char **) malloc (MAX_DIR_ENTRIES*sizeof (char *));

  for (i=0; i<MAX_DIR_ENTRIES; i++)
    flist[i] = (char *) malloc (256);

}

static int input_plugin_open (char *mrl) {

  char *filename;

  printf ("input dvd : input_plugin_open >%s<\n",mrl);

  /*
   * do we handle this kind of MRL ?
   */

  if (strncmp (mrl, "dvd://",6))
    return 0;

  filename = &mrl[6];

  printf ("input dvd : input_plugin_open media type correct. file name is %s\n",
	  filename);

  file_lbstart = openDVDFile (filename, &file_size) ;
  file_lbcur = file_lbstart;

  if (!file_lbstart) {
    printf ("unable to find >%s< on dvd.\n",filename);
    return 0;
  }

  file_size_left = file_size;
 
  return 1 ;
}

static uint32_t input_plugin_read (char *buf, uint32_t nlen) {

  if (nlen != DVD_VIDEO_LB_LEN) {
    printf ("ERROR in input_dvd plugin read: %d bytes is not a sector!\n",nlen);
    return 0;
  }

  if (file_size_left < nlen)
    return 0;

  if (read (file_fd, buf, DVD_VIDEO_LB_LEN)) {

    file_lbcur++;
    file_size_left -= DVD_VIDEO_LB_LEN;

    return DVD_VIDEO_LB_LEN;
  } else
    printf ("read error in input_dvd plugin\n");

  return 0;
}

static uint32_t input_plugin_seek (uint32_t offset, int origin) {

  offset /= DVD_VIDEO_LB_LEN;

  switch (origin) {
  case SEEK_END:
    offset = (file_size / DVD_VIDEO_LB_LEN) - offset;

  case SEEK_SET:
    file_lbcur = file_lbstart + offset;
    file_size_left = file_size - (offset * DVD_VIDEO_LB_LEN);
    break;
  case SEEK_CUR:
    file_lbcur += origin;
    file_size_left = file_size - ((file_lbcur - file_lbstart) * DVD_VIDEO_LB_LEN);
    break;
  default:
    printf ("error in input dvd plugin seek:%d is an unknown origin\n",origin);
  }

  return lseek (file_fd, file_lbcur * (off_t) DVD_VIDEO_LB_LEN, SEEK_SET);
}

static uint32_t input_plugin_get_length (void) {
  return file_size;
}

static uint32_t input_plugin_get_capabilities (void) {
  return INPUT_CAP_SEEKABLE | INPUT_CAP_BLOCK;
}

static uint32_t input_plugin_get_blocksize (void) {
  return DVD_VIDEO_LB_LEN;
}

static void input_plugin_close (void) {
 UDFCloseDisc (); 
}

static char *input_plugin_get_identifier (void) {
  return "DVD";
}

static int input_plugin_get_dir (char *filename, char **flist, int *nEntries) {


  int i, filenumber;

  if (filename)
    return 0;

  if ((filenumber=UDFOpenDisc(DVD))>=0) {

    int    nFiles;
    char **filelist = (char **) malloc (MAX_DIR_ENTRIES*sizeof (char *));

    for (i=0; i<MAX_DIR_ENTRIES; i++)
      filelist[i] = (char *) malloc (256);

    UDFListDir ("/VIDEO_TS",MAX_DIR_ENTRIES,filelist,&nFiles);

    *nEntries = 0;
    for (i=0; i<nFiles; i++) {
      int nLen = strlen (filelist[i]);

      if (nLen<4) 
	continue;
      
      if (strcasecmp (&filelist[i][nLen-4], ".VOB")) {
	sprintf (flist[*nEntries], "dvd://%s\n",filelist[i]);

	*nEntries++;
      }

    }

    for (i=0; i<MAX_DIR_ENTRIES; i++)
      free(filelist[i]);

    UDFCloseDisc (); 

  } else {
    *nEntries = 0;
  }

  return 1;
}

static input_plugin_t plugin_op = {
  NULL,
  NULL,
  input_plugin_init,
  input_plugin_open,
  input_plugin_read,
  input_plugin_seek,
  input_plugin_get_length,
  input_plugin_get_capabilities,
  input_plugin_get_dir,
  input_plugin_get_blocksize,
  input_plugin_close,
  input_plugin_get_identifier
};

input_plugin_t *input_plugin_getinfo(void) {

  return &plugin_op;
}
